import azure.cognitiveservices.speech as speechsdk
import os

class AzureProvider:
    def __init__(
        self, 
        subscription_key=os.getenv("AZURE_KEY"), 
        region=os.getenv("AZURE_REGION")
    ):
        self.valid = True
        self.value_error = None
        try:
            if not subscription_key:
                self.value_error = "Azure subscription key not provided. Set the AZURE_KEY environment variable."
                raise ValueError("Azure subscription key not provided. Set the AZURE_KEY environment variable.")
            if not region:
                self.value_error = "Azure region not provided. Set the AZURE_REGION environment variable."
                raise ValueError("Azure region not provided. Set the AZURE_REGION environment variable.")
            self.subscription_key = subscription_key
            self.region = region
        except ValueError as e:
            self.valid = False

    def ensure_valid(self):
        if not self.valid:
            raise ValueError(self.value_error)

    def synthesis(
        self, 
        text,
        voice_name=None,
        language="en-US"
    ):
        self.ensure_valid()
        speech_config = speechsdk.SpeechConfig(
            subscription=self.subscription_key, 
            region=self.region
        )
        speech_config.speech_synthesis_language = language
        if voice_name is not None:
            speech_config.speech_synthesis_voice_name = voice_name
        
        audio_config = None

        speech_synthesizer = speechsdk.SpeechSynthesizer(
            speech_config=speech_config,
            audio_config=audio_config
        )

        result = speech_synthesizer.speak_text_async(text).get()

        if result.reason == speechsdk.ResultReason.SynthesizingAudioCompleted:
            return result.audio_data
        elif result.reason == speechsdk.ResultReason.Canceled:
            cancellation_details = result.cancellation_details
            print(f"Speech synthesis canceled: {cancellation_details.reason}")
            if cancellation_details.reason == speechsdk.CancellationReason.Error:
                print(f"Error details: {cancellation_details.error_details}")
                raise ValueError(f"Error details: {cancellation_details.error_details}")
            raise ValueError(f"Speech synthesis canceled: {cancellation_details.reason}")

    def available_voices(self):
        voices = []
        speech_config = speechsdk.SpeechConfig(
            subscription=self.subscription_key, 
            region=self.region
        )

        speech_synthesizer = speechsdk.SpeechSynthesizer(speech_config=speech_config)

        result = speech_synthesizer.get_voices_async().get()

        if result.reason == speechsdk.ResultReason.VoicesListRetrieved:
            for voice in result.voices:
                voice_object = {
                    'name': voice.name,
                    'locale': voice.locale,
                    'gender': voice.gender,
                    'localName': voice.local_name,
                }
                voices.append(voice_object)
            return voices
        elif result.reason == speechsdk.ResultReason.Canceled:
            cancellation_details = result.cancellation_details
            print(f"Speech synthesis canceled: {cancellation_details.reason}")
            if cancellation_details.reason == speechsdk.CancellationReason.Error:
                print(f"Error details: {cancellation_details.error_details}")
                raise ValueError(f"Error details: {cancellation_details.error_details}")
            raise ValueError(f"Speech synthesis canceled: {cancellation_details.reason}")        



import unittest
from unittest.mock import patch
import os
from supertts import SuperTTS

class TestOpenAIProvider(unittest.TestCase):
    # @patch('supertts.providers.openai_provider.speak')
    # def test_azure_speak(self, mock_speak):
    #     """Test that OpenAI's speak function is called correctly."""
    #     res=openai_provider.speak("Hello world", model="tts-1", voice="nova")
    #     mock_speak.assert_called_once_with("Hello world", model="tts-1", voice="nova")

    @unittest.skipIf(os.getenv('AZURE_KEY') is None, "Azure key not set.")
    def test_azure_voices(self):
        """Test that Azure available_voices function is called correctly."""
        supertts = SuperTTS(provider="azure")
        voices = supertts.voices()

        # check if voices length is more than 0
        self.assertGreater(len(voices), 0, "No voices returned")

if __name__ == '__main__':
    unittest.main()




def save_audio(response, filename):
    """
    Saves the audio response to a file.

    :param response: The audio response from tts.tts().
    :param filename: The filename to save the audio to.
    """
    if isinstance(response, bytes):
        audio_data = response
    # Check if the response is from OpenAI and needs additional handling
    elif hasattr(response, 'read'):
        # This assumes the response object has a read() method to get bytes
        audio_data = response.read()
    else:
        raise TypeError("Unsupported response type for saving audio.")

    with open(filename, 'wb') as f:
        f.write(audio_data)


from openai import OpenAI
import os

DEFAULT_MODEL = "tts-1"
DEFAULT_VOICE = "alloy"

VOICES = [
            {
                "name": "alloy",
                "gender": "male"
            },
            {
                "name": "echo",
                "gender": "male"
            },
            {
                "name": "fable",
                "gender": "male"
            },
            {
                "name": "onyx",
                "gender": "male"
            },
            {
                "name": "nova",
                "gender": "female"
            },
            {
                "name": "shimmer",
                "gender": "male"
            },
        ]

class OpenAIProvider:
    def __init__(
        self,
        api_key=os.getenv("OPENAI_API_KEY")
    ):
        self.valid = True
        self.valid_error = None
        self.voices = VOICES
        try:
            if api_key is None or api_key == "":
                raise EnvironmentError('The environment variable OPENAI_API_KEY is not set. Please set it to your OpenAI API key.')
            self.client = OpenAI(
                api_key=api_key
            )
        except EnvironmentError as e:
            self.valid = False
            self.valid_error = "The environment variable OPENAI_API_KEY is not set. Please set it to your OpenAI API key."

    def ensure_valid(self):
        if not self.valid:
            raise ValueError(self.valid_error)

    def synthesis(
        self,
        text,
        model: str = DEFAULT_MODEL,
        voice: str = DEFAULT_VOICE
    ):
        self.ensure_valid()
        model = DEFAULT_MODEL if model is None else model
        voice = DEFAULT_VOICE if voice is None else voice
        voices = self.available_voices()
        if model not in ["tts-1", "tts-1-hd"]:
            raise ValueError(f"Invalid model: {model}")
        if voice not in [voice["name"] for voice in voices]:
            raise ValueError(f"Invalid voice: {voice}")
        response = self.client.audio.speech.create(
            model=model,
            voice=voice,
            input=text
        )
        return response
    
    def available_voices(
        self
    ):
        return self.voices



import unittest
from unittest.mock import patch
from supertts import SuperTTS
import os

class TestOpenAIProvider(unittest.TestCase):
    # @patch('supertts.providers.openai_provider.speak')
    # def test_openai_speak(self, mock_speak):
    #     """Test that OpenAI's speak function is called correctly."""
    #     res=openai_provider.speak("Hello world", model="tts-1", voice="nova")
    #     mock_speak.assert_called_once_with("Hello world", model="tts-1", voice="nova")

    @patch.dict(os.environ, {"OPENAI_API_KEY": "test_value"})
    def test_openai_voices(self):
        """Test that OpenAI's available_voices function is called correctly."""
        supertts = SuperTTS()
        voices = supertts.voices()
        expected_voices = [
            {"name": "echo", "gender": "male"},
            {"name": "alloy", "gender": "male"},
            {"name": "fable", "gender": "male"},
            {"name": "onyx", "gender": "male"},
            {"name": "nova", "gender": "female"},
            {"name": "shimmer", "gender": "male"},
        ]

        voices_tuples = [tuple(voice.items()) for voice in voices]
        expected_voices_tuples = [tuple(voice.items()) for voice in expected_voices]

        self.assertEqual(set(voices_tuples), set(expected_voices_tuples), "Available voices do not match expected")

if __name__ == '__main__':
    unittest.main()



import unittest
from supertts import SuperTTS

class TestCoreFunctionality(unittest.TestCase):
    def test_tts_without_provider(self):
        """Test that tts function raises ValueError when no provider is found."""
        supertts = SuperTTS()
        with self.assertRaises(ValueError):
            supertts.tts("Hello world", provider="nonexistent")

    def test_voices_without_provider(self):
        """Test that voices function raises ValueError when no provider is found."""
        supertts = SuperTTS()
        with self.assertRaises(ValueError):
            supertts.voices(provider="nonexistent")

if __name__ == '__main__':
    unittest.main()



from supertts.providers.azure_provider import AzureProvider
from supertts.providers.openai_provider import OpenAIProvider
from supertts.providers.google_provider import GoogleProvider
from supertts.constants import PROVIDERS

class SuperTTS:
    def __init__(self, provider='openai'):
        self.provider = provider
        self.openai = OpenAIProvider()
        self.azure = AzureProvider()
        self.google = GoogleProvider()

    def tts(
        self, 
        text, 
        provider=None, 
        voice=None, 
        model=None
    ):
        """
        Converts text to speech using the specified voice and model.

        :param text: The text to convert to speech.
        :param provider: The TTS provider to use. (openai, google, azure)
        :param voice: The voice to use for the TTS.
        :param model: The TTS model to use.
        :return: An audio stream or file.
        """
        provider = provider or self.provider
        if provider == PROVIDERS["OPENAI"]:
            return self.openai.synthesis(text, voice=voice, model=model)
        elif provider == PROVIDERS["GOOGLE"]:
            return self.google.synthesis(text)
        elif provider == PROVIDERS["AZURE"]:
            return self.azure.synthesis(text)
        else:
            raise ValueError(f"Unsupported provider: {provider}")

    def voices(self, provider=None):
        """
        Lists available voices for the current TTS provider.

        :param provider: The TTS provider to use. (openai, google, azure)
        :return: A list of available voices.
        """
        provider = provider or self.provider
        if provider == PROVIDERS["OPENAI"]:
            tts = self.openai
        elif provider == PROVIDERS["GOOGLE"]:
            tts = self.google
        elif provider == PROVIDERS["AZURE"]:
            tts = self.azure
        else:
            raise ValueError(f"Unsupported provider: {provider}")
        return tts.available_voices()
